// Programmed using the knockoff Pro Micro boards - use Arduino/Genuino Micro as board type
// Watch out - your board's name might change and you'll have to seelct it again in Tools/Port 
// after you program it the first time

// Libraries 
// You need to install all three libraries - search for them under Tools/Manage Libraries, and then 
// search for the word in front of the .h
#include <elapsedMillis.h>
#include <Bounce2.h>
#include <MIDIUSB.h>

// Set up synonyms to make writing the code easier and more logical
#define Button 10       // Button pin
#define LED 9           // LED use for signalling the state of the board
#define Pressure A0     // Gauge Pressure analog pin

// Instantiate Bounce objects for the buttons
// this reduces the noise present when closing a switch
Bounce modeButton = Bounce(); 

// Timers used to avoid sending way too may messages over the MIDI bus
elapsedMillis timeElapsed; 
elapsedMillis noteTimer; 
elapsedMillis noteDecay; 

// MIDI Information
int midichannel = 0; // MIDI Channel - zero referenced, so for Channel 1, set as 0.
int midiCC = 74; // CC # to be used to send CC messages

// Note Information
// Pitch defines notes using MIDI note numbers - 60 is middle C, valid pitches are 0-127
// 0 is super low,and 127 is super high! Be wise about the pitches you include!
byte pitch[] = {60, 64, 67, 71, 72}; // You can change which pitch plays by changing these MIDI Note Numbers
int numNotes = 5; // this variable MUST match the number of notes in the pitch array in the previous line
int currPitch = 0;
int noteState = 0;
int decayTime = 100;

// Pressure values
int prevPval = 0;
int currPval = 0;
int CCPval = 0;
int sensitivity = 5;

// Pressure Data Variables - these will change depending on your balloon size and how hard you press it
// Mode 2 below will let you reset the High Pressure value, so it doesn't have to be perfect
// The low pressure (lowP) is set during the start up loop, so don't squeeze your balloon when turning it on
int lowP;
int highP = 650;

// Create a "mode" variable that allows for the system to be used
// Mode 0: Send CC and Note messages
// Mode 1: Send CC messages only
// Mode 2: Set the low and high pressure values
int mode = 0;

// The setup loop runs once when the board powers on
void setup() {
  // Setup the button and debouncer:
  pinMode(Button,INPUT_PULLUP);
  modeButton.attach(Button);
  modeButton.interval(5); // interval in ms
  pinMode(LED, OUTPUT);
  analogWrite(LED,10);
  pinMode(Pressure, INPUT);
  lowP = analogRead(Pressure);
}

// the main loop runs over and over...
void loop() {
  updateButtons();
  if(mode == 0) // Mode 0 - send notes and CC messages
  {
    checkNote();
    checkCC();
  } else if(mode == 1) // Mode 1 - send notes and CC messages
  {
    checkCC();
  } else // Mode 2 - set lower and upper pressure limits
  {
    setPressure();
  }
}

// The rest of these are functions being called in the "main loop" above
void updateButtons(){
  modeButton.update();
  if(modeButton.fell()){
    mode = (mode + 1) % 3;
    if(mode ==2){
      highP = 0;
      lowP = 1023;
    }
    analogWrite(LED,mode*80+10);
  }
}

void checkCC(){
  if(timeElapsed>20){
    // Map the output value between 30 and 110 - the largest range you could use is 0 - 127
    int val = constrain(map(analogRead(Pressure),lowP,highP,30, 110), 30,110);
    sendCC(midichannel, midiCC, val);
    timeElapsed = 0; 
  }  
}

void checkNote() {
  if(noteTimer > 20){
    noteTimer = 0;
    prevPval = currPval;
    currPval = analogRead(Pressure);
    int diff = currPval-prevPval;
    if((diff > sensitivity) && (noteState == 0)){
      currPitch = random(0,numNotes-1);
      noteOn(midichannel, pitch[currPitch], random(90,120));
      noteState = 1;
      noteDecay = 0;
    }
    if((noteDecay > decayTime) && (noteState == 1)){
      noteOff(midichannel, pitch[currPitch], 0);
      noteState = 0;   
    }    
  }
}

void setPressure(){
  int Pval = analogRead(Pressure);
  if(Pval > highP){
    highP = Pval;
  }
  if(Pval < lowP){
    lowP = Pval;
  }
}

// A function to make it easier to send control change messages
void sendCC(byte channel, byte CC, byte value) {
  midiEventPacket_t CCdata = {0x0B, 0xB0 | channel, CC, value};
  MidiUSB.sendMIDI(CCdata);
  MidiUSB.flush();
}

// A function to make it easier to send Note On messages
void noteOn(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOn = {0x09, 0x90 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOn);
  MidiUSB.flush();
}

// A function to make it easier to send Note Off messages
void noteOff(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOff = {0x08, 0x80 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOff);
  MidiUSB.flush();
}
